@@ -5,54 +5,51 @@ use strict;
 use warnings;
 use Carp qw(croak);
 use vars qw($VERSION);
-$VERSION = 0.05;
+$VERSION = 0.06;
+use Math::Cephes qw(:dists);
 use Statistics::Lite qw(:all);
 use Statistics::Descriptive;
 use Statistics::Distributions;
-use String::Util qw(hascontent); 
+use String::Util qw(nocontent); 
+use Scalar::Util qw(looks_like_number);
 
 =head1 NAME
 
-Statistics::Zed - Basic ztest/zscore, with optional continuity correction
+Statistics::Zed - Basic deviation ratio: observed less expected (with optional continuity correction) divided by root variance 
 
 =head1 SYNOPSIS
 
- use Statistics::Zed 0.05;
+ use Statistics::Zed 0.06;
 
  $zed = Statistics::Zed->new(
-    ccorr    => 1,
+    ccorr    => 1, 
     tails    => 2,
     precision_s => 5,
     precision_p => 5,
  );
 
- ($z_value, $p_value, $observed_deviation, $standard_deviation) = 
-    $zed->score( # or zscore
-        observed => $obs,
-        expected => $exp, # or key stdev or variance
-        error => $variance,
- );
-
- $deviate = $zed->test( # or ztest
-        observed => $obs,
-        expected => $exp,
-        error => $variance, # or key stdev or variance
-        samplings => $samplings,
+ ($z_value, $p_value, $observed_deviation, $standard_deviation) = $zed->score( # or 'zscore'
+    observed => $obs,
+    expected => $exp,
+    variance => $variance, # or 'stdev'
  );
 
  $p_value = $zed->z2p(value => $z_value, tails => 1|2);
- 
  $z_value = $zed->p2z(value => $p_value, tails => 1|2);
 
 =head1 DESCRIPTION
 
-Calculates a standard, run-of-the-mill z-score: the ratio of an observed deviation to a standard deviation; and performs a z-test or returns the z-score. Purpose is simply to support L<Statistics::Sequences|Statistics::Sequences>.
++ Calculates a standard, run-of-the-mill z-score: the ratio of an observed deviation to a standard deviation.
+
++ Provides wraps to convert z-value to p-value, and convert p-value to z-value.
+
++ Organizes accumulated observed, expected and variance values over two or more runs ahead of making the calculation.
+
++ Purpose is to support tests in  L<Statistics::Sequences|Statistics::Sequences>.
 
 =head1 METHODS
 
-=head2 Interface
-
-=head3 new
+=head2 new
 
  $zed = Statistics::Zed->new();
 
@@ -60,9 +57,7 @@ Returns a Statistics::Zed object. Accepts setting of any of the OPTIONS.
 
 =cut
 
-#-----------------------------------------------
 sub new {
-#-----------------------------------------------
     my $class = shift;
     my $args = ref $_[0] ? $_[0] : {@_};
 	my $self = {};
@@ -77,63 +72,17 @@ sub new {
             $self->{$_} = $args->{$_};
         }
     }
-
 	return $self;
 }
 
-=head2 Stats
+=head2 zscore
 
-=head3 ztest
-
- $zed->ztest(observed => 'number', expected => 'number', variance => 'number (non-zero)', samplings => 'positive integer')
-
-I<Alias>: test
-
-You supply the C<observed> and c<expected> values of your statistic, and the C<variance> or C<stdev> or just C<error> of any kind; ... and the number of C<samplings> ("sample-size", I<N>, "trials", etc.).
-
-Optionally specify a logical value for L<ccorr|ccorr> for performing the continuity-correction to the observed deviation, and a value of either 1 or 2 to specify the L<tails|tails> for reading off the probability associated with the I<z>-value. When passed to this function, values of L<ccorr|ccorr> and L<tails|tails> become the values used in this and subsequent tests.
-
-When called in array context, returns an array consisting of the z-statistic ("standard normal deviate"), its probability, the observed deviation (the difference between the observed and expected values of your statistic), and the standard deviation (the square-root of the variance supplied).
-
-If you only want the I<z>-value, then expect a string:
-
- $z_value = $zed->test(...)
-
-The basic formula is the basic:
-
-=for html <p>&nbsp;&nbsp;&nbsp;<i>Z<sub><o>&times;</o></sub></i> = ( <i><o>&times;</o></i> &ndash; <i>&micro;</i> ) / SD / &not;/<i>n</i></p>
-
-=cut
-
-#-----------------------------------------------
-sub ztest {
-#-----------------------------------------------
-    my $self = shift;
-    my $args = ref $_[0] ? $_[0] : {@_};
-    $self->{$_} = $args->{$_} foreach keys %{$args};
-    my ($z, $p, $obs_dev, $exp_dev) = ();
-    croak "Need to define observed ($self->{'observed'}) and expected ($self->{'expected'}) values for ztest" if !hascontent($self->{'observed'}) || !hascontent($self->{'expected'});
-
-    $obs_dev = $self->{'observed'} - $self->{'expected'};
-    $obs_dev = _ccorr($obs_dev) if $self->{'ccorr'};
-    $exp_dev = _exp_dev($self, 1);
-    return undef if !$exp_dev;
-    $z = $obs_dev / $exp_dev;
- 
-    $p = $self->z2p(value => $z, tails => $self->{'tails'});
-    $z = sprintf('%.' . $self->{'precision_s'} . 'f', $z) if $self->{'precision_s'};
-
-    return wantarray ? ($z, $p, $obs_dev, $exp_dev) : $z;
-}
-*test = \&ztest;
-
-=head3 zscore
-
- $zed->zscore(observed => 'number', expected => 'number', variance => 'number (non-zero)')
+ $zval = $zed->zscore(observed => 'number', expected => 'number', variance => 'number (non-zero)')
+ ($zval, $pval, $obs_dev, $stdev) = $zed->zscore(observed => 'number', expected => 'number', variance => 'number (non-zero)')
 
 I<Alias>: score
 
-You supply the C<observed> and C<expected> values of your statistic, and the C<variance> or C<stdev> or just C<error> of any kind.
+You supply the C<observed> and C<expected> values of your statistic, and the C<variance> or C<stdev>.
 
 Optionally specify a logical value for L<ccorr|ccorr> for performing the continuity-correction to the observed deviation, and a value of either 1 or 2 to specify the L<tails|tails> for reading off the probability associated with the I<z>-value. When passed to this function, values of L<ccorr|ccorr> and L<tails|tails> become the values used in this and subsequent tests.
 
@@ -151,46 +100,44 @@ where I<X> is the expected value (mean, etc.).
 
 =cut
 
-#-----------------------------------------------
 sub zscore {
-#-----------------------------------------------
     my $self = shift;
 	my $args = ref $_[0] ? $_[0] : {@_};
     $self->{$_} = $args->{$_} foreach keys %{$args};
-    my ($z, $p, $obs_dev, $exp_dev) = ();
-    croak "Need to define observed ($self->{'observed'}) and expected ($self->{'expected'}) values for zscore" if !hascontent($self->{'observed'}) || !hascontent($self->{'expected'});
+    croak "Need to define observed ($self->{'observed'}) and expected ($self->{'expected'}) values for zscore" if nocontent($self->{'observed'}) || nocontent($self->{'expected'});
 
+    my ($z, $p, $obs_dev, $exp_dev) = ();
     $obs_dev = $self->{'observed'} - $self->{'expected'};
     $obs_dev = _ccorr($obs_dev) if $self->{'ccorr'};
-    $exp_dev = _exp_dev($self, 0);
-    return undef if !$exp_dev;
+    $exp_dev = _exp_dev($self);
+    return 0 if !$exp_dev;
+
     $z = $obs_dev / $exp_dev;
- 
     $p = $self->z2p(value => $z, tails => $self->{'tails'});
     $z = sprintf('%.' . $self->{'precision_s'} . 'f', $z) if $self->{'precision_s'};
-    
     return wantarray ? ($z, $p, $obs_dev, $exp_dev) : $z;
 }
 *score = \&zscore;
 
-=head3 z2p
+=head2 z2p
 
+ $p = $zed->z2p($z); # assumes 2-tailed
  $p = $zed->z2p(value => $z); # assumes 2-tailed
  $p = $zed->z2p(value => $z, tails => 1);
 
 I<Alias>: C<p_value>
 
-Send a I<z>-value, get its associated I<p>-value, 2-tailed by default, or depending on what the value of $zed->{'tails'} is, or what is sent as the second argument, if anything.
+Send a I<z>-value, get its associated I<p>-value, 2-tailed by default, or depending on what the value of $zed->{'tails'} is, or what is sent as the second argument, if anything. If you send just one value (unkeyed), it is taken as the value.
 
 =cut
 
-#-----------------------------------------------
 sub z2p {
-#-----------------------------------------------
     my $self = shift;
-    my $args = ref $_[0] ? $_[0] : {@_};
+    my $args = ref $_[0] ? $_[0] : scalar(@_) > 1 ? {@_} : {value => shift};
     $args->{'tails'} ||= 2;
-	my $p = Statistics::Distributions::uprob (abs($args->{'value'}));
+    return '' if nocontent($args->{'value'}) or !looks_like_number($args->{'value'});
+    return 1 if $args->{'value'} == 0;
+	my $p = Statistics::Distributions::uprob(abs($args->{'value'}));
     $p *= 2 if $args->{'tails'} == 2;
 	$p = Statistics::Distributions::precision_string($p);
     $p = 1 if $p > 1;
@@ -200,35 +147,41 @@ sub z2p {
 }
 *p_value = \&z2p;
 
-=head3 p2z
+=head2 p2z
 
+ $z_value = $zed->p2z($p) # the p-value is assumed to be 2-tailed
  $z_value = $zed->p2z(value => $p) # the p-value is assumed to be 2-tailed
- $z_value = $zed->p2z(value => $p, tails => 1) # 1-tailed probability
+ $z_value = $zed->p2z(value => $p, tails => 1) # specify 1-tailed probability
 
-Returns the I<z>-value associated with a I<p>-value, using L<Math::Cephes|Math::Cephes> C<ndtri> function ("phi"). If the C<tails> attribute equals 2 (indicating a 2-tailed probability value), the I<p>-value is firstly divided by 2. A check is firstly made to ensure that what is sent as a probability actually looks like a probability.
+Returns the I<z>-value associated with a I<p>-value using L<Math::Cephes|Math::Cephes> C<ndtri> ("phi"). B<The p-value is assumed to be two-tailed>, and so is firstly (before conversion) divided by 2, e.g., .05 becomes .025 so you get I<z> = 1.96.  As a one-tailed probability, it is then assumed to be a probability of being I<greater> than a certain amount, i.e., of getting a I<z>-value I<greater> than that observed. So the phi function is actually given (1 - I<p>-value) to work on - so you get back the I<z>-value up to that point, before the I<p>-value has to fit in. So .055 comes back as 1.598 (speaking of the top-end of the distribution), and .991 comes back as -2.349 (now going from right to left across the distribution). These assumptions are not the same as found in inversion methods in common spreadsheet packages but seem expected by human users like me of I<z>-values and their I<p>-values. (complain if useful)
 
 =cut
 
-#-----------------------------------------------
-sub p2z { # This utility might be moved to a Statistics::Convert module later ...
-#-----------------------------------------------
+sub p2z {
     my $self = shift;
-    my $args = ref $_[0] ? $_[0] : {@_};
-    my $p_val = $args->{'value'};
-    croak __PACKAGE__, "::p2z The value sent as a p-value ($p_val) does not appear to be valid" if !_valid_p($p_val);
-    $args->{'tails'} ||= 2;
-    my $z_val;
+    my ($p_val, $z_val, $tails) = (); 
+    if (scalar(@_) > 1) {
+        my $args = ref $_[0] ? $_[0] : {@_};
+        $p_val = $args->{'value'};
+        $tails = $args->{'tails'} || 2;
+    }
+    else {
+        $p_val = shift;
+        $tails = 2;
+    }
+    #return undef if !_valid_p($p_val);
+    #croak __PACKAGE__, "::p2z The value sent as a p-value (" . ($p_val || '') . ") does not appear to be valid" if !_valid_p($p_val);
+
     # Avoid ndtri errors by first accounting for 0 and 1 ...
-    if (!$p_val) {
+    if (!$p_val or $p_val < 0) {
         $z_val = '';
     }
-    elsif ($p_val == 1)  {
+    elsif ($p_val >= 1)  {
         $z_val = 0;
     }
     else {
-        $p_val /= 2 if $args->{'tails'} == 2; # p-value should be for one tail of the distribution
-        require Math::Cephes;
-        $z_val = Math::Cephes::ndtri(1 - $p_val); # for the area within the p_value'd tails
+        $p_val /= 2 if $tails == 2; # p-value has been given as two-tailed - we only use one side
+        $z_val = ndtri(1 - $p_val); # 
     }
     return $z_val;
 }
@@ -254,49 +207,38 @@ Prints to STDOUT a line giving the z_value and p_value.
 =cut
 
 sub dump {
-    my ($self, $series) = @_;
+    my $self = shift;
     print "Z = $self->{'z_value'}, $self->{'tails'}p = $self->{'p_value'}\n";
 }
 
 # Calc expected deviation, ensuring valid denominator:
-sub _exp_dev {
-    my ($self) = @_;
-    my $exp_dev;
-    my $stdev = $self->{'stdev'} ? 
+sub _exp_dev { # ugly elongated thing :
+    my $self = shift;
+    return ($self->{'stdev'} and $self->{'stdev'} > 0) ? 
       $self->{'stdev'} : 
-        $self->{'variance'} ?
+        ($self->{'variance'} and $self->{'variance'} > 0) ?
             sqrt($self->{'variance'}) :
-            $self->{'error'} ?
-                $self->{'error'} :
-                    croak "Need a standard deviation or variance value for z-testing";##return undef;##
-    if ($self->{'test'}) {
-        croak "Need non-zero and positive number of samplings/units" if ! $self->{'samplings'} or $self->{'samplings'} < 0;
-        $exp_dev = $stdev / sqrt($self->{'samplings'});
-    }
-    else {
-        $exp_dev = $stdev;
-    }
-    return $exp_dev;
+               0;#croak "Need a standard deviation or variance value for z-testing";##return undef;##
 }
 
-=head2 Series testing
+=head1 Series testing
 
-A means to aggregate results from multiple tests is supported. Three methods are presently used to effect this.
+Aggregate results from multiple tests using a method to initialise the series, another to add data to the series, and a final one to sum the data, compute an aggregate Zscore from them, with all the values of interest ready to dump. L<Statistics::Descriptive|Statistics::Descriptive> objects are kept of each series' observed, expected and variance values, as sent to L<series_update>, and a separate count of the number of series is kept - $zed->{'series'}->{'count'}.
 
-=head3 series_init
+=head2 series_init
 
 Clears any already accumulated data from previous tests.
 
-=head3 series_update
+=head2 series_update
 
  $zed->series_update() # use any cached values of "observed", "expected" and "variance"
  $zed->series_update(variance => 'number', expected => 'number', observed => 'number') # supply own in a hash
 
-Called once you have performed a test on a sample. It caches the observed, expectation and variance values from the test.
+Called once you have performed a test on a sample. It caches the observed, matchcount_expected and variance values from the test.
 
-=head3 series_test
+=head2 series_test
 
-Sums the observed, expectation and variance values from all the tests updated to the series since calling L<series_init|series_init>, and produces a I<z>-value from these sums. It returns nothing in particular, but the following statement shows how the series values can be accessed.
+Sums the observed, matchcount_expected and variance values from all the tests updated to the series since calling L<series_init|series_init>, and produces a I<z>-value from these sums. Returns the same values as C<zscore>: wanting an array, then the I<z>-value, its probability, the observed deviation and the standard deviation; otherwise, the I<z>-value alone. Additionally, the C<$zed->{'series'}> object hash is lumped with the usual values, so you can access them like so:
 
  print "Series summed runs: 
     expected = ", $zed->{'series'}->{'expected'}, " 
@@ -305,19 +247,15 @@ Sums the observed, expectation and variance values from all the tests updated to
 
 =cut
 
-#-----------------------------------------------
 sub series_init {
-#-----------------------------------------------
     my ($self, %args) = @_;
-    $self->{'series_stat'}->{$_} = Statistics::Descriptive::Sparse->new() foreach qw/observed expected variance z_value/;
-    $self->{'series'}->{$_} = 0 foreach qw/observed expected variance z_value p_value /;
+    $self->{'series_stat'}->{$_} = Statistics::Descriptive::Sparse->new() foreach qw/observed expected variance samplings z_value/;
+    $self->{'series'}->{$_} = 0 foreach qw/observed expected variance samplings z_value p_value count/;
    # $self->{'series'}->{'p_value'} = 1;
   # $self->{'series'}->{'expected'} = 1;
 }
 
-#-----------------------------------------------
 sub series_update {
-#-----------------------------------------------
     my $self = shift;
     my $args = ref $_[0] ? $_[0] : {@_};
     if (! defined $args->{'variance'}) {
@@ -325,39 +263,52 @@ sub series_update {
             $args->{'variance'} = $args->{'stdev'}**2;
         }
         else {
-            croak "No variance offered in series_update for calculating deviation";
+            croak "No variance in series_update for calculating deviation";
         }
     }
-	foreach (qw/observed expected variance z_value/) {
+	foreach (qw/observed expected variance samplings z_value/) {
     	$self->{'series_stat'}->{$_}->add_data($args->{$_}) if defined $args->{$_};
 	}
+    $self->{'series'}->{'count'}++;
 }
 
-#-----------------------------------------------
 sub series_test {
-#-----------------------------------------------
-    my ($self) = (shift);
-    croak 'No data for series-testing appear to have been loaded; maybe you need to call series_update with some data' if ! $self->{'series_stat'}->{'variance'}->count();
+    my ($self, @args) = (shift, @_);
+    return  if ! $self->{'series_stat'}->{'variance'}->count();
+   # croak 'No data for series-testing appear to have been loaded; maybe you need to call series_update with some data' if ! $self->{'series_stat'}->{'variance'}->count();
     my $o = $self->{'series_stat'}->{'observed'}->sum();
     my $e = $self->{'series_stat'}->{'expected'}->sum();
     my $v = $self->{'series_stat'}->{'variance'}->sum();
-    my ($z, $pz, $obs_dev, $stdev) = $self->zscore(
+    my ($z, $pz, $obs_dev, $exp_dev) = $self->zscore(
         observed => $o,
         expected => $e,
-        variance => $v);
+        variance => $v,
+        @args,
+    );
     $self->{'series'}->{'observed'} = $o;
     $self->{'series'}->{'expected'} = $e;
     $self->{'series'}->{'z_value'} = $z;# || 0;
     $self->{'series'}->{'p_value'} = $pz;# || 1;
     $self->{'series'}->{'obs_dev'} = $obs_dev;
-    $self->{'series'}->{'stdev'} = $stdev;
+    $self->{'series'}->{'stdev'} = $exp_dev;
     $self->{'series'}->{'variance'} = $v;
-    $self->{'series'}->{'samplings'} = $self->{'series_stat'}->{'variance'}->count();
+    $self->{'series'}->{'samplings'} = $self->{'series_stat'}->{'samplings'}->sum();
 	if ($self->{'series_stat'}->{'z_value'}->count()){
     	$self->{'series'}->{'stouffer_z'} = $self->{'series_stat'}->{'z_value'}->sum() / sqrt($self->{'series_stat'}->{'z_value'}->count());
     	$self->{'series'}->{'stouffer_p'} = $self->p_value($self->{'series'}->{'stouffer_z'});
 	}
-    return $self;
+    return wantarray ? ($z, $pz, $obs_dev, $exp_dev) : $z;
+}
+
+=head2 series_str
+
+Returns string: line giving the z_value and p_value for the series.
+
+=cut
+
+sub series_str {
+    my ($self) = @_;
+    return "Z (N = $self->{'series'}->{'samplings'}) = $self->{'series'}->{'z_value'}, $self->{'tails'}p = $self->{'series'}->{'p_value'}";
 }
 
 =head2 series_dump
@@ -367,18 +318,15 @@ Prints to STDOUT a line giving the z_value and p_value for the series.
 =cut
 
 sub series_dump {
-    my ($self) = @_;
-    print "Z (N = $self->{'series'}->{'samplings'}) = $self->{'series'}->{'z_value'}, $self->{'tails'}p = $self->{'series'}->{'p_value'}\n";
+    print series_str(@_), "\n";
 }
 
-#-----------------------------------------------
 sub _valid_p {
-#-----------------------------------------------
     my $p = shift;
-    return (($p !~ /^0?\.\d+$/) && ($p !~ /^\d+?\.[\de-]+/ )) || ($p < 0 || $p > 1) ? $p == 1 ? 1 : 0 : 1;
+    return undef if nocontent($p) or !looks_like_number($p);
+    return ( ($p !~ /^0?\.\d+$/) && ($p !~ /^\d+?\.[\de-]+/ )) || ($p < 0 || $p > 1) ? $p == 1 ? 1 : 0 : 1;
 }
 
-
 1;
 __END__
 
@@ -388,7 +336,7 @@ The following can be set in the call to C<new> or C<test> or C<score>.
 
 =head2 ccorr
 
-Apply the continuity correction. Default = 0.
+Apply the continuity correction, Stetigkeitskorrektur. Default = 0.
 
 =head2 tails
 
